#include <pthread.h>
#include <math.h>
#include "main.h"
#include "ptpu.h"

int pbus[NMAX_CAMS], pdev[NMAX_CAMS];
int nCams = 0;
pthread_t threads[NMAX_CAMS];
int usPerPic = 5000000;

/**
 * Search for cameras, return the bus/dev numbers for the first n
 */
int find_cameras(int n, int *pbusn, int *pdevn)
{
    int iCam=0, busn, devn;

    if (n<1 || !pbusn || !pdevn) return 0;

    for(busn=1; busn<16; busn++)
    {
        for(devn=1; devn<256; devn++)
        {
            if (find_device(busn,devn))
            {
                info("Found camera %d: bus=%d, dev=%d\n", iCam+1, busn, devn);
                pbusn[iCam] = busn;
                pdevn[iCam] = devn;
                iCam++;
                if (iCam == n) return n;
            }
        }
    }

    return iCam;
}

/**
 * Search the first 16 USB buses and the first 256 devices on each bus
 * for a camera.
 */
void scan_for_cameras()
{
    int busn, devn;
    int nFound = 0;

    for(busn=1; busn<16; busn++)
    {
        for(devn=1; devn<256; devn++)
        {
            if (find_device(busn,devn))
            {
                print(" Found device: bus=%d, dev=%d\n", busn, devn);
                nFound++;
            }
        }
    }
    print("Found %d cameras.\n", nFound);
}

/**
 * Run the given camera command by searching for the appropriate handler
 */
int run_cam_cmd(int iCam, const char* cmd, const char* arg)
{
    cmd_t *c;
    
    c = cmds;
    while ( c->command )
    {
	    if ( ! strcmp(cmd, c->command) ) 
	    {
            if (c->handler) return c->handler(iCam, c, arg);
            else
            {
                debug("FIXME: no c->handler for %s!\n", cmd);
                exit(1);
            }
	    }
	    c++;
    }

    debug("Error: unknown command in run_cam_cmd: %s\n", cmd);
    return 0;
}

/**
 * Returns the number of microseconds between a and b (ie. a-b)
 */
int usdiff(struct timeval* a, struct timeval* b)
{
    const int x = 2147483647;
    const int xus = x / 1000000;
    int s, us;
    
    if (!a || !b)
    {
        debug("Error: null param to usdiff (%p, %p)\n", a, b);
        return 0;
    }

    /* handle the seconds field */
    s = a->tv_sec - b->tv_sec;
    if (s > xus) return x;
    if (s < -xus) return -x;
    s *= 1000000;

    /* handle the usec field */
    us = a->tv_usec - b->tv_usec;

    return s + us;    
}

/**
 * Convert the timeval into a timestamp string
 */
int getTimeStamp(struct timeval *time, char* ts)
{
    time_t sec;
    struct tm stm;
    
    if (!time || !ts)
    {
        debug("Error: null param to getTimeStamp (%p, %p)\n", time, ts);
        return 0;
    }

    sec = time->tv_sec;
    if (time->tv_usec >= 500000) sec++;
    if (!gmtime_r(&sec, &stm))
    {
        debug("Error: failed to convert seconds to tm struct (%d)\n", sec);
        return 0;
    }

    sprintf(ts, "%04d%02d%02d-%02d%02d%02d", stm.tm_year+1900, stm.tm_mon+1,
            stm.tm_mday, stm.tm_hour, stm.tm_min, stm.tm_sec);
    
    return 1;
}

/**
 * Thread function for starting and capturing from the cameras
 */
void* cam_thread_func(void* arg)
{
    int iCam = (int)arg;
    int iCap = 1;
    int iPrevCam = (iCam==0 ? nCams-1 : iCam-1);
    int delta;
    char ts[256];
    struct timeval timeStart, timeEnd;

    /* start the cameras */
    info("Starting camera %d...\n", iCam+1);
    if (!start_camera(iCam))
    {
        debug("failed.\n");
        return NULL;
    }

    /* TODO: capture dummy image; this seems to let the rest of setup work properly */
    capture_file(iCam, "foo%02d.jpg", iCam+1);

    /* setup the camera */
    info("Setting up camera %d...\n", iCam);
    run_cam_cmd(iCam, CMD_TVAV, "aperture priority");
    run_cam_cmd(iCam, CMD_APERTURE, "2.8");
    run_cam_cmd(iCam, CMD_ISO_SPEED, "Auto");
    run_cam_cmd(iCam, CMD_SHUTTER, "auto");
    run_cam_cmd(iCam, CMD_WHITE_BAL, "auto");
    run_cam_cmd(iCam, CMD_FP, "auto");
    focuslock(iCam, 0);

    run_cam_cmd(iCam, CMD_EXPCOMP, "0");
    run_cam_cmd(iCam, CMD_PHOTO_EFFECT, "off");
    run_cam_cmd(iCam, CMD_FLASH, "off");
    run_cam_cmd(iCam, CMD_MACRO, "off");
    run_cam_cmd(iCam, CMD_ZOOM, "0");
    run_cam_cmd(iCam, CMD_QUAL, "normal");
    run_cam_cmd(iCam, CMD_SIZE, "large");

    /* wait a bit to space out the cameras */
    usleep(usPerPic/nCams * iCam);
    info("Starting capture from camera %d...\n", iCam+1);
    
    /* now we can start capturing images */
    while(settings[iCam].bStarted)
    {
        gettimeofday(&timeStart, NULL);
        delta = usdiff(&timeStart, &settings[iPrevCam].timeLastPic);
        delta = usPerPic / nCams - delta;
        if (delta > 0)
        {
            debug("delaying cam %d for %.4fs\n", iCam, delta/1000000.0);
            usleep(delta);
        }

        gettimeofday(&timeStart, NULL);
        settings[iCam].timeLastPic = timeStart;
        if (getTimeStamp(&timeStart, ts))
            capture_file(iCam, "oqo-cam%02d-%04d-%s.jpg", iCam+1, iCap, ts);
        else{
            strcpy(ts, "err");
            capture_file(iCam, "oqo-cam%02d-%04d.jpg", iCam+1, iCap);
        }
        gettimeofday(&timeEnd, NULL);
        delta = usdiff(&timeEnd, &timeStart);
        /*debug("Time to capture image (%d @ %s): %.4fs\n", iCam+1, ts, delta/1000000.0);*/
        usPerPic = (int)(usPerPic*.9 + delta*.1);
        /*debug("New average cap time (%d): %.4fs\n", iCam+1, usPerPic/1000000.0);*/
        iCap++;
    }

    return (void*)1;
}

int run_cameras()
{
    int i, ret;

    /* init settings to not started state */
    for(i=0; i<NMAX_CAMS; i++)
    {
        settings[i].bStarted = 0;
        gettimeofday(&settings[i].timeLastPic, NULL);
    }

    /* locate cameras on the USB */
    nCams = find_cameras(NMAX_CAMS, pbus, pdev);
    if (nCams < 1)
    {
        print("Error: no cameras!\n");
        return 0;
    }

    /* create a thread for each camera */
    for(i=0; i<nCams; i++)
    {
        ret = pthread_create(&threads[i], NULL, cam_thread_func, (void*)i);
        if (ret != 0)
        {
            print("Error: failed to start camera thread %d (ret: %d)\n", i+1, ret);
            return 0;
        }
    }

    /* wait for all of the camera threads to die */
    for(i=0; i<nCams; i++) pthread_join(threads[i], NULL);
    
    quit();    
    return 1;
}
