/* capture.c (c) 2004 Petr Danecek
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <unistd.h>
#include <ctype.h>
#include "main.h"
#include "ptpu.h"
#include "props.h"

settings_t settings[NMAX_CAMS];

#undef CHK_STARTED
#define CHK_STARTED(x) {if (!settings[x].bStarted) { debug("Error: camera %d not started\n", x); return 0;} }

#undef RUN_N_CHK
#define RUN_N_CHK(x) { int stat = (x); if((stat)!=PTP_RC_OK) { \
      debug("bad status on %s:%d - %04X\n", __FILE__, __LINE__, stat); \
      return 0; } }


cmd_t cmds[] = 
{
    { CMD_APERTURE,  PTP_DPC_CANON_Aperture, prop16_handler, aperture_vals, "Set the aperture"},
    { CMD_EXPCOMP, PTP_DPC_CANON_ExpCompensation, prop8_handler, exposure_compensation_vals,"Exposure compensation"},
    { CMD_PHOTO_EFFECT, PTP_DPC_CANON_PhotoEffect, prop16_handler, photo_effect_vals, "Photo effect" },
    { CMD_FLASH,  PTP_DPC_CANON_FlashMode, prop8_handler, flash_vals, "Set flash mode" },
    { CMD_FP,   PTP_DPC_CANON_FocusingPoint, prop16_handler, focuspoint_vals, "Sets the focusing point to center/auto" },
    { CMD_HELP, 0,  NULL,NULL, "More help on a commmand: \"help COMMAND\""},
    { CMD_ISO_SPEED, PTP_DPC_CANON_ISOSpeed, prop16_handler, iso_speed_vals, "ISO speed" },
    { CMD_MACRO,   PTP_DPC_CANON_MacroMode, prop8_handler, macro_vals, "Set the macro mode" }, 
    { CMD_LIGHT_METER, PTP_DPC_CANON_MeteringMode, prop8_handler, light_metering_vals, "Light metering mode" },
    { CMD_SHUTTER, PTP_DPC_CANON_ShutterSpeed, prop16_handler, shutter_vals,  "Set the shutter speed" },
    { CMD_TVAV, PTP_DPC_CANON_TvAvSetting, prop8_handler, tvav_vals,"TV/Av Setting - sets aperture/shutter priority"},
    { CMD_WHITE_BAL, PTP_DPC_CANON_WhiteBalance, prop8_handler, white_balance_vals, "White balance"},
    { CMD_ZOOM,   PTP_DPC_CANON_Zoom, prop16_handler, zoom_vals, "Set zoom: 0..6 optical zoom, 7..10 digital zoom"},
    { CMD_QUAL,   PTP_DPC_CANON_ImageQuality, prop8_handler, qual_vals, "Set image quality"},
    { CMD_SIZE,   PTP_DPC_CANON_ImageSize, prop8_handler, size_vals, "Set image size"},
    { NULL, 0, NULL, NULL, NULL },
};



/**
 * start a particular camera
 */
int start_camera(int iCam)
{
    int i;

    if ( settings[iCam].bStarted ) 
    {
	    debug("error: camera %d already started\n", iCam+1);
	    return 1;
    }
    
    for (i=0; i<5; i++)
    {
	    if (i > 0) 
	    {
            debug("Trying again\n");
            sleep(1);
	    }
        
	    settings[iCam].dev = find_device(pbus[iCam],pdev[iCam]);

	    if ( ! settings[iCam].dev )
	    {
            debug("no device matching bus=%d/dev=%d found\n", pbus[iCam],pdev[iCam]);
            continue;
	    }

	    find_endpoints(&settings[iCam]);
	    if ( ! ptp_usb_init(&settings[iCam]) ) 
	    {
            debug("could not init ptp_usb\n");
	    }

	    sleep(1);

	    if ( (ptp_opensession(&(settings[iCam].params),1)) != PTP_RC_OK )
	    {
            debug("could not open session\n");
            ptp_usb_close(&settings[iCam]);
            continue;
	    }

	    break;
    }

    if (i == 5) 
    {
	    debug("error: could not initialize camera %d", iCam);
	    return 0;
    }
      
    RUN_N_CHK( magic_code(&settings[iCam]) );
    RUN_N_CHK( ptp_canon_startshootingmode(&(settings[iCam].params)) );
    RUN_N_CHK( usb_checkevent_wait(&settings[iCam]) );
    ptp_checkevent(&settings[iCam],NULL);
    ptp_checkevent(&settings[iCam],NULL);
    RUN_N_CHK( set_prop16(&settings[iCam], PTP_DPC_CANON_D029, 3) );

    settings[iCam].bStarted = 1;
    return 1;
}

/**
 * close the cameras
 */
int quit()
{
    int i;

    info("Closing cameras...\n");
    for(i=0; i<nCams; i++)
    {
        if (!settings[i].bStarted) continue;
        settings[i].bStarted = 0;
        info(" closing camera %d... ", i+1);
        camera_close(&settings[i]);
        info("done.\n");
    }
    nCams = 0;

    return 0;
}

int prop16_handler(int iCam, const cmd_t *cmd, const char *args)
{
    int prop;
    CHK_STARTED(iCam);
      
    prop = value_ok(cmd->params, args);

    if ( prop == -1 )
    {
        debug("cmd=%s -- bad parameter \"%s\"", cmd->command, args);
	    return 0;
    }

    RUN_N_CHK( set_prop16(&settings[iCam], cmd->ptpcode, prop) );

    return 1;
}


int prop8_handler(int iCam, const cmd_t *cmd, const char *args)
{
    int prop;
    CHK_STARTED(iCam);
      
    prop = value_ok(cmd->params, args);

    if ( prop == -1 )
    {
	    debug("cmd=%s -- bad parameter \"%s\"", cmd->command, args);
	    return 0;
    }

    RUN_N_CHK( set_prop8(&settings[iCam], cmd->ptpcode, prop) );
    return 1;
}

int capture_file(int iCam, const char *fmt, ...)
{
    int n;
    char fname[512];

    va_list ap;
    va_start(ap, fmt);
    vsprintf(fname, fmt, ap);
    va_end(ap);

    CHK_STARTED(iCam);
    if (!fname) return 0;
    n = capture(&settings[iCam], (char*)fname);
	    
    return ( n == PTP_RC_OK );
}

int focuslock(int iCam, int bLock)
{
    CHK_STARTED(iCam);


    if (bLock) lock_focus(&settings[iCam]);
    else unlock_focus(&settings[iCam]);
      
    return 1;
}

