#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif

#include "main.h"

int DEBUG = 0;
int INFO = 1;

void debug(const char *format, ...)
{
    va_list ap;
    
    if (!DEBUG) return;
    va_start(ap, format);
    vfprintf(stderr, format, ap);
    va_end(ap);
    fflush(stderr);
}

void info(const char *format, ...)
{
    va_list ap;
    
    if (!INFO) return;
    va_start(ap, format);
    vfprintf(stderr, format, ap);
    va_end(ap);
    fflush(stderr);
}

void print(const char *format, ...)
{
    va_list ap;
    va_start(ap, format);
    vprintf(format, ap);
    va_end(ap);
    fflush(stdout);
}

/**
 * run a shall command (e.g., "mkdir -p /foo/bar")
 * this function will block until the command has finished
 */
int run_shell_cmd(char *cmdline)
{
    pid_t pid;
    char* args[3];
    args[0] = "bash";
    args[1] = "-c";
    args[2] = cmdline;

    pid = fork();
    if (pid < 0)
    {
        fprintf(stderr, "Error: failed to fork for command exec\n");
        return 0;
    }

    if (pid > 0)
    {
        /* wait for child (the exec) to finish */
        int status;
        waitpid(pid, &status, 0);
        fprintf(stderr, "Child process completed (%d).\n", status);
        return 1;
    }
    else{
        fprintf(stderr, "Child: executing \"/bin/bash -c %s\"\n", cmdline);
        execvp("/bin/bash", args);
        fprintf(stderr, "Error: exec failed (%d) (%s)\n", errno, cmdline);
        return 0;
    }        
}

/**
 * handle ctrl-c
 */
void sigterm_handler(int signal, struct sigcontext sc)
{
    quit();
    print("\nFinished exit procedures.  Camera should shut shutdown in a few seconds.\n");
    print("Turn off camera if shutdown does not complete successfully...\n\n");
    exit(1);
}

/**
 * Return the index of the given option if it is found, otherwise 0
 */
int checkOpt(int argc, char **argv, char *name)
{
    int i;
    for(i=1; i<argc; i++)
    {
        if (strcasecmp(argv[i], name)==0) return i;
    }
    return 0;
}

/**
 * Scan the argv list for a string of the form "name=X" and return X
 * as an int.  Return 'def' if the name isn't found.
 */
int getIntOpt(int argc, char **argv, char *name, int def)
{
    int i, n, nlen;

    if (!argv || !name) return def;
    nlen = strlen(name);
    
    for(i=1; i<argc; i++)
    {
        n = strlen(argv[i]);
        if (n < nlen+2) continue;
        if (strncmp(argv[i], name, nlen)==0 && argv[i][nlen]=='=')
            return atoi(&argv[i][nlen+1]);
    }

    return def;
}

/**
 * Scan the argv list for a string of the form "name=X" and return X
 * as a boolean (1 or 0).  The option is considered true if X=Y,T, or
 * True, regardless of case.  Return 'def' if the name isn't found.
 */
int getBoolOpt(int argc, char **argv, char *name, int def)
{
    int i, n, nlen;

    if (!argv || !name) return def;
    nlen = strlen(name);
    
    for(i=1; i<argc; i++)
    {
        n = strlen(argv[i]);
        if (n < nlen+2) continue;
        if (strncmp(argv[i], name, nlen)==0 && argv[i][nlen]=='=')
        {
            char *opt = &argv[i][nlen+1];
            return (strcasecmp(opt, "T")==0 || strcasecmp(opt, "Y")==0
                    || strcasecmp(opt, "YES")==0 || strcasecmp(opt, "TRUE")==0);
        }
    }

    return def;
}

int main(int argc, char **argv)
{
    /* see if the user wants help */
    if (checkOpt(argc, argv, "help") || checkOpt(argc, argv, "-?"))
    {
        fprintf(stderr, "USAGE: assist-cap [options]\n");
        fprintf(stderr, " help            this help message\n");
        fprintf(stderr, " scan            scan for cameras\n");
        fprintf(stderr, " debug           enable verbose debug spew\n");
        fprintf(stderr, " info=Y/N        info output (def: Y)\n");
        fprintf(stderr, "\n");
        return 0;
    }

    /* isntall sigint handler */
    signal(SIGINT, (void (*)(int))sigterm_handler);

    /* check other params */
    DEBUG = checkOpt(argc, argv, "debug");
    INFO = getBoolOpt(argc, argv, "info", INFO);
    
    /* are we doing a scan or actually driving the cameras? */
    if (checkOpt(argc, argv, "scan")) /* device scan? */
    {
        print("Scanning for cameras...\n");
        scan_for_cameras();
    }
    else run_cameras();

    return 0;
}
